@@ -27,15 +27,14 @@ func registerAndVerifyJob(s *Server, t *testing.T, prefix string, counter int) *
 }
 
 func TestSearch_PrefixSearch_Job(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	prefix := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -61,15 +60,14 @@ func TestSearch_PrefixSearch_Job(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_ACL(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	jobID := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s, root := TestACLServer(t, func(c *Config) {
+	s, root, cleanupS := TestACLServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 	state := s.fsm.State()
@@ -174,15 +172,14 @@ func TestSearch_PrefixSearch_ACL(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_All_JobWithHyphen(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	prefix := "example-test-------" // Assert that a job with more than 4 hyphens works
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -221,15 +218,14 @@ func TestSearch_PrefixSearch_All_JobWithHyphen(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_All_LongJob(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	prefix := strings.Repeat("a", 100)
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -268,15 +264,14 @@ func TestSearch_PrefixSearch_All_LongJob(t *testing.T) {
 
 // truncate should limit results to 20
 func TestSearch_PrefixSearch_Truncate(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	prefix := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -305,15 +300,15 @@ func TestSearch_PrefixSearch_Truncate(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_AllWithJob(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	prefix := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
 
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -345,13 +340,13 @@ func TestSearch_PrefixSearch_AllWithJob(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_Evals(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -382,13 +377,13 @@ func TestSearch_PrefixSearch_Evals(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_Allocation(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -427,13 +422,13 @@ func TestSearch_PrefixSearch_Allocation(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_All_UUID(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -479,13 +474,13 @@ func TestSearch_PrefixSearch_All_UUID(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_Node(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -520,13 +515,13 @@ func TestSearch_PrefixSearch_Node(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_Deployment(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -557,13 +552,14 @@ func TestSearch_PrefixSearch_Deployment(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_AllContext(t *testing.T) {
-	assert := assert.New(t)
 	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	assert := assert.New(t)
+
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
 
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -607,16 +603,14 @@ func TestSearch_PrefixSearch_AllContext(t *testing.T) {
 
 // Tests that the top 20 matches are returned when no prefix is set
 func TestSearch_PrefixSearch_NoPrefix(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
-
 	prefix := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -644,16 +638,14 @@ func TestSearch_PrefixSearch_NoPrefix(t *testing.T) {
 // Tests that the zero matches are returned when a prefix has no matching
 // results
 func TestSearch_PrefixSearch_NoMatches(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
-
 	prefix := "aaaaaaaa-e8f7-fd38-c855-ab94ceb8970"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -678,17 +670,16 @@ func TestSearch_PrefixSearch_NoMatches(t *testing.T) {
 // Prefixes can only be looked up if their length is a power of two. For
 // prefixes which are an odd length, use the length-1 characters.
 func TestSearch_PrefixSearch_RoundDownToEven(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
 	id1 := "aaafaaaa-e8f7-fd38-c855-ab94ceb89"
 	id2 := "aaafeaaa-e8f7-fd38-c855-ab94ceb89"
 	prefix := "aaafa"
 
-	t.Parallel()
-	s := TestServer(t, func(c *Config) {
+	s, cleanupS := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 	})
-
-	defer s.Shutdown()
+	defer cleanupS()
 	codec := rpcClient(t, s)
 	testutil.WaitForLeader(t, s.RPC)
 
@@ -714,22 +705,21 @@ func TestSearch_PrefixSearch_RoundDownToEven(t *testing.T) {
 }
 
 func TestSearch_PrefixSearch_MultiRegion(t *testing.T) {
+	t.Parallel()
 	assert := assert.New(t)
-
 	jobName := "exampleexample"
 
-	t.Parallel()
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 		c.Region = "foo"
 	})
-	defer s1.Shutdown()
+	defer cleanupS1()
 
-	s2 := TestServer(t, func(c *Config) {
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.NumSchedulers = 0
 		c.Region = "bar"
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 
 	TestJoin(t, s1, s2)
 	testutil.WaitForLeader(t, s1.RPC)
