@@ -15,12 +15,13 @@ import (
 
 func TestNomad_JoinPeer(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "region2"
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 
 	testutil.WaitForResult(func() (bool, error) {
@@ -56,12 +57,13 @@ func TestNomad_JoinPeer(t *testing.T) {
 
 func TestNomad_RemovePeer(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, nil)
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, nil)
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.Region = "global"
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	TestJoin(t, s1, s2)
 
 	testutil.WaitForResult(func() (bool, error) {
@@ -95,32 +97,34 @@ func TestNomad_RemovePeer(t *testing.T) {
 
 func TestNomad_ReapPeer(t *testing.T) {
 	t.Parallel()
+
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
-	s1 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.NodeName = "node1"
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node1")
 	})
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.NodeName = "node2"
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node2")
 	})
-	defer s2.Shutdown()
-	s3 := TestServer(t, func(c *Config) {
+	defer cleanupS2()
+	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.NodeName = "node3"
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node3")
 	})
-	defer s3.Shutdown()
+	defer cleanupS3()
 	TestJoin(t, s1, s2, s3)
 
 	testutil.WaitForResult(func() (bool, error) {
@@ -189,30 +193,31 @@ func TestNomad_ReapPeer(t *testing.T) {
 
 func TestNomad_BootstrapExpect(t *testing.T) {
 	t.Parallel()
+
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node1")
 	})
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node2")
 	})
-	defer s2.Shutdown()
-	s3 := TestServer(t, func(c *Config) {
+	defer cleanupS2()
+	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node3")
 	})
-	defer s3.Shutdown()
+	defer cleanupS3()
 	TestJoin(t, s1, s2, s3)
 
 	testutil.WaitForResult(func() (bool, error) {
@@ -255,13 +260,13 @@ func TestNomad_BootstrapExpect(t *testing.T) {
 
 	// Join a fourth server after quorum has already been formed and ensure
 	// there is no election
-	s4 := TestServer(t, func(c *Config) {
+	s4, cleanupS4 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node4")
 	})
-	defer s4.Shutdown()
+	defer cleanupS4()
 
 	// Make sure a leader is elected, grab the current term and then add in
 	// the fourth server.
@@ -301,32 +306,33 @@ func TestNomad_BootstrapExpect(t *testing.T) {
 
 func TestNomad_BootstrapExpect_NonVoter(t *testing.T) {
 	t.Parallel()
+
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	s1 := TestServer(t, func(c *Config) {
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node1")
 		c.NonVoter = true
 	})
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node2")
 		c.NonVoter = true
 	})
-	defer s2.Shutdown()
-	s3 := TestServer(t, func(c *Config) {
+	defer cleanupS2()
+	s3, cleanupS3 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node3")
 	})
-	defer s3.Shutdown()
+	defer cleanupS3()
 	TestJoin(t, s1, s2, s3)
 
 	// Assert that we do not bootstrap
@@ -342,13 +348,13 @@ func TestNomad_BootstrapExpect_NonVoter(t *testing.T) {
 	})
 
 	// Add the fourth server that is a voter
-	s4 := TestServer(t, func(c *Config) {
+	s4, cleanupS4 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 2
 		c.DevMode = false
 		c.DevDisableBootstrap = true
 		c.DataDir = path.Join(dir, "node4")
 	})
-	defer s4.Shutdown()
+	defer cleanupS4()
 	TestJoin(t, s1, s2, s3, s4)
 
 	testutil.WaitForResult(func() (bool, error) {
@@ -409,16 +415,17 @@ func TestNomad_BootstrapExpect_NonVoter(t *testing.T) {
 
 func TestNomad_BadExpect(t *testing.T) {
 	t.Parallel()
-	s1 := TestServer(t, func(c *Config) {
+
+	s1, cleanupS1 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 2
 		c.DevDisableBootstrap = true
 	})
-	defer s1.Shutdown()
-	s2 := TestServer(t, func(c *Config) {
+	defer cleanupS1()
+	s2, cleanupS2 := TestServer(t, func(c *Config) {
 		c.BootstrapExpect = 3
 		c.DevDisableBootstrap = true
 	})
-	defer s2.Shutdown()
+	defer cleanupS2()
 	servers := []*Server{s1, s2}
 	TestJoin(t, s1, s2)
 
