@@ -1,11 +1,12 @@
-import { none } from '@ember/object/computed';
 import { computed } from '@ember/object';
-import { alias } from '@ember/object/computed';
+import { alias, none, and } from '@ember/object/computed';
 import Fragment from 'ember-data-model-fragments/fragment';
 import attr from 'ember-data/attr';
 import { fragment, fragmentOwner, fragmentArray } from 'ember-data-model-fragments/attributes';
 
 export default Fragment.extend({
+  allocation: fragmentOwner(),
+
   name: attr('string'),
   state: attr('string'),
   startedAt: attr('date'),
@@ -13,11 +14,15 @@ export default Fragment.extend({
   failed: attr('boolean'),
 
   isActive: none('finishedAt'),
+  isRunning: and('isActive', 'allocation.isRunning'),
 
-  allocation: fragmentOwner(),
-  task: computed('allocation.taskGroup.tasks.[]', function() {
+  isConnectProxy: computed('task.kind', function() {
+    return (this.get('task.kind') || '').startsWith('connect-proxy:');
+  }),
+
+  task: computed('name', 'allocation.taskGroup.tasks.[]', function() {
     const tasks = this.get('allocation.taskGroup.tasks');
-    return tasks && tasks.findBy('name', this.get('name'));
+    return tasks && tasks.findBy('name', this.name);
   }),
 
   driver: alias('task.driver'),
@@ -40,6 +45,18 @@ export default Fragment.extend({
       failed: 'is-error',
     };
 
-    return classMap[this.get('state')] || 'is-dark';
+    return classMap[this.state] || 'is-dark';
   }),
+
+  restart() {
+    return this.allocation.restart(this.name);
+  },
+
+  ls(path) {
+    return this.store.adapterFor('task-state').ls(this, path);
+  },
+
+  stat(path) {
+    return this.store.adapterFor('task-state').stat(this, path);
+  },
 });
